#!/usr/bin/perl
#
# Author  : Yann Stettler
# Version : 1.0
#
# Desc :
# Query ping server for collected data about a host.
#
# Arguments:
#
# <host>   = Host we want to get information about
# <server> = Optional IP/hostname of the ping server to query. If omitted,
#            will check if a server is configured for this host else will use
#            the default server.
#
# Packages needed:
# IO::Socket
# IO::Socket::Timeout
# Data::Validate::Domain
# Data::Validate::IP
#
use strict;
use warnings;
use IO::Socket;
use IO::Socket::Timeout;
use Errno qw(ETIMEDOUT EWOULDBLOCK);
use Data::Validate::Domain qw(is_domain);
use Data::Validate::IP qw(is_ipv4 is_ipv6);


#############################################################################
# Configuration                                                             #
#############################################################################
my $SERVER='127.0.0.1';              # Address of default ping server
my $PORT=9928;                       # Port of ping server
my $SECRET="12345";                  # Must match secret in ping server config

# Define address of the ping server to use for a given host (if different
# from default server). ('host' => 'ping server')

my %PingServer = ( );

#############################################################################

my $host=shift(@ARGV);

if (!$host) {
   die "Missing host argument\n";
}

my $pingsrv=shift(@ARGV);
if (!$pingsrv || (!is_domain($pingsrv) && !is_ipv4($pingsrv))) {
   $pingsrv=$PingServer{$host};
   if (!$pingsrv) {
      $pingsrv=$SERVER;
   }
}

open(OUT, ">> /tmp/fping.log");
print OUT "Host: $host Server: $pingsrv\n";
close(OUT);

$| = 1;

my $socket = new IO::Socket::INET (
    PeerHost => $pingsrv,
    PeerPort => $PORT,
    Proto => 'tcp',
    Timeout => 2,
) or die "Cannot connect to the server $!\n";

IO::Socket::Timeout->enable_timeouts_on($socket);
$socket->read_timeout(3);
$socket->write_timeout(3);

$socket->send("$SECRET\n$host\n");
shutdown($socket, 1);
my $result=$socket->getline;
if (! $result || (! $result && ( 0+$! == ETIMEDOUT || 0+$! == EWOULDBLOCK ))) {
   shutdown($socket, 1);
   die "Timeout getting data from server\n";
}
$socket->close();

$result=~s/\s+$//;
if ($result ne "") {
  my ($tmp, $data)=split(/ /, $result, 2);
  print $data;
} else {
  # No data returned for this host. Probably mean no data collected yet
  print "min: avg: max: dev: loss:100.00";
}

exit;
